@@ -1,4 +1,4 @@
-import requests, json, time, threading, sys
+import requests, json, time, threading, sys, copy
 
 UpdateInfo = type("UpdateInfo", (), {})
 
@@ -11,6 +11,7 @@ class ConversationInfo:
         self.info_object = None
         self.thread = None
         self.error_handler = error_handler
+        self.reply_keyboard = False
 
     def set_error_handler(self, new_handler):
         self.error_handler = new_handler
@@ -154,33 +155,31 @@ class TelegramBot:
         self._cancel_command = command
 
     def set_cancel(self, command: str, function):
+        del self.commands[self._cancel_command]
         self._cancel_command = command
         self._cancel_func = function
+        self.commands[command] = function
 
     def get_updates(self):
         try:
             args = dict()
             if self._last_update:
                 args["offset"] = self._last_update + 1
-            pure_result = self._send_request("getUpdates", args=args)
+            pure_result = self._send_request("getUpdates", args=args, timeout=10)
             if pure_result["ok"]:
                 for result in pure_result["result"]:
                     self._last_update = result["update_id"]
                 return pure_result["result"]
             return None
-        except:
-            print("[WARNING] Bot cannot reach API (could be caused by temporary timeout)")
-            return None
+        except requests.exceptions.Timeout:
+            return requests.exceptions.ConnectionError
+        except requests.exceptions.ConnectionError:
+            return requests.exceptions.ConnectionError
 
-    def _send_request(self, function, args=dict()):
+    def _send_request(self, function, args=dict(), timeout=10):
         base = "https://api.telegram.org/bot"
         url = f"{base}{self.token}/{function}"
-        if args:
-            url += "?"
-            for key, item in args.items():
-                url += f"{key}={item}&"
-            url = url[:-1]
-        response = requests.get(url)
+        response = requests.get(url, timeout=timeout, params=args)
         return json.loads(response.content)
 
     def set_command(self, command, function):
@@ -205,14 +204,25 @@ class TelegramBot:
                 return True
         return False
 
-    def send_message(self, chat_id: str, msg: str, parse_mode=None) -> bool:
+    def send_message(self, chat_id: str, msg: str, parse_mode=None, remove_reply_keyboard=True) -> bool:
         if parse_mode == None:
             parse_mode = self._standard_parse_mode
-        result = self._send_request("sendMessage", {
-            "chat_id": str(chat_id),
-            "text": str(msg),
-            "parse_mode": str(parse_mode)
-        })
+        def split_into_parts(str, part_len):
+            out = list()
+            while str:
+                temp = str[:part_len]
+                out.append(temp)
+                str = str[len(temp):]
+            return out
+        for temp_msg in split_into_parts(str(msg), 4000):
+            args = {
+                "chat_id": str(chat_id),
+                "text": temp_msg,
+                "parse_mode": str(parse_mode)
+            }
+            if remove_reply_keyboard:
+                args["reply_markup"] = json.dumps({"remove_keyboard": True})
+            result = self._send_request("sendMessage", args)
         if result["ok"]:
             return True
         return False
@@ -254,11 +264,12 @@ class TelegramBot:
             return True
         return False
 
-    def _wait_input(self, chat_id, msg, timeout=None, interval=0.3):
+    def _wait_input(self, chat_id, *msgs, timeout=None, interval=0.3):
         if not timeout:
             timeout = self._standart_timeout
-        if msg:
-            self.send_message(chat_id, msg)
+        for msg in msgs:
+            if msg != None:
+                self.send_message(chat_id, msg)
         conv_infos = self._get_infos(chat_id)
         conv_infos.awaiting_input = True
         start_time = time.time()
@@ -280,16 +291,43 @@ class TelegramBot:
         if typing:
             self._send_request("sendChatAction", {"chat_id": chat_id,"action": "typing"})
         time.sleep(seconds)
-    
+
     def _user_selection(self, chat_id, msg: str, options: list, columns=1, timeout=None, interval=0.3):
         if not timeout:
             timeout = self._standart_timeout
+        c_options = copy.deepcopy(options)
         buttons = list()
-        while len(options) > 0:
+        while len(c_options) > 0:
             button_row = list()
             for i in range(columns):
-                if len(options):
-                    option = str(options.pop(0))
+                if len(c_options):
+                    option = str(c_options.pop(0))
+                    button = {"text": option}
+                    button_row.append(button)
+            buttons.append(button_row)
+        json_buttons = json.dumps({
+            "keyboard": buttons,
+            "one_time_keyboard": True
+        })
+        output = self._send_request("sendMessage", {
+            "chat_id": chat_id,
+            "text": msg,
+            "reply_markup": json_buttons
+        })
+        conv_infos = self._get_infos(chat_id)
+        conv_infos.reply_keyboard = True
+        return self._wait_input(chat_id, None, timeout=timeout, interval=0.3)
+    
+    def _user_inline_selection(self, chat_id, msg: str, options: list, columns=1, timeout=None, interval=0.3):
+        if not timeout:
+            timeout = self._standart_timeout
+        c_options = copy.deepcopy(options)
+        buttons = list()
+        while len(c_options) > 0:
+            button_row = list()
+            for i in range(columns):
+                if len(c_options):
+                    option = str(c_options.pop(0))
                     button = {"text": option, "callback_data": option}
                     button_row.append(button)
             buttons.append(button_row)
@@ -302,22 +340,25 @@ class TelegramBot:
         return self._wait_input(chat_id, None, timeout=timeout, interval=0.3)
 
     def _make_info_object(self, message_dict):
+        print(message_dict)
         ui = UpdateInfo()
         ui.message_id = str(message_dict["message_id"])
-        ui.username = "@" + message_dict["chat"]["username"]
+        ui.username = "@" + message_dict["from"]["username"]
         ui.text = message_dict["text"]
-        ui.first_name = message_dict["chat"]["first_name"]
+        ui.first_name = message_dict["from"]["first_name"]
         ui.date = message_dict["date"]
         ui.chat_id = message_dict["chat"]["id"]
         ui.from_bot = message_dict["from"]["is_bot"]
         ui.send_message = self.send_message
-        ui.respond = lambda msg, **args: self.send_message(ui.chat_id, msg, **args)
-        ui.input = lambda msg, **args: self._wait_input(ui.chat_id, msg, **args)
+        ui.respond = lambda *args, **kwargs: self.send_message(ui.chat_id, *args, **kwargs)
+        ui.input = lambda *args, **kwargs: self._wait_input(ui.chat_id, *args, **kwargs)
         ui.get_answer = self._wait_input
         ui.stop_thread = lambda: self._stop_thread(ui.chat_id)
-        ui.wait = lambda seconds, **args: self._wait_type(ui.chat_id, seconds, **args)
-        ui.select = lambda msg, options, **args: self._user_selection(ui.chat_id, msg, options, **args)
+        ui.wait = lambda *args, **kwargs: self._wait_type(ui.chat_id, *args, **kwargs)
+        ui.select = lambda *args, **kwargs: self._user_selection(ui.chat_id, *args, **kwargs)
         ui.select_from = self._user_selection
+        ui.inline_select = lambda *args, **kwargs: self._user_inline_selection(ui.chat_id, *args, **kwargs)
+        ui.inline_select_from = self._user_inline_selection
         ui.input_from = self._wait_input
         ui.bot = self
 
@@ -352,32 +393,39 @@ class TelegramBot:
 
     def handle_message_update(self, update):
         self._temp_interaction_count += 1
-        info_object = self._make_info_object(update["message"])
-        text = update["message"]["text"]
-        trimmed_text = text[:10] + ("..." if len(text) > 10 else "")
-        found_command = False
-        if text.startswith("/") and len(text) > 1:
-            if self._run_command(text[1:], update["message"]):
-                self.log(f"Chat {info_object.chat_id} called /{trimmed_text[1:]}", USER)
-                found_command = True
-        if not(found_command):
-            conv_infos = self._get_infos(info_object.chat_id)
-            if conv_infos.awaiting_input:
-                self.log(f"Received Answer (\"{trimmed_text}\") from {info_object.chat_id}", USER)
-                conv_infos.awaiting_answer = info_object.text
-                conv_infos.awaiting_input = False
-                return
-            elif self._private_func:
-                self._run_user_func(self._private_func, info_object)
-                self.log(f"Received \"{trimmed_text}\" from {info_object.chat_id}", USER)
-            else:
-                self.log(f"No Normal Message Handler defined", ERROR)
+        try:
+            info_object = self._make_info_object(update["message"])
+            text = update["message"]["text"]
+            trimmed_text = text[:10] + ("..." if len(text) > 10 else "")
+            found_command = False
+            if text.startswith("/") and len(text) > 1:
+                if self._run_command(text[1:], update["message"]):
+                    self.log(f"Chat {info_object.chat_id} called /{trimmed_text[1:]}", USER)
+                    found_command = True
+            if not(found_command):
+                conv_infos = self._get_infos(info_object.chat_id)
+                if conv_infos.awaiting_input:
+                    self.log(f"Received Answer (\"{trimmed_text}\") from {info_object.chat_id}", USER)
+                    conv_infos.awaiting_answer = info_object.text
+                    conv_infos.awaiting_input = False
+                    return
+                elif self._private_func:
+                    self._run_user_func(self._private_func, info_object)
+                    self.log(f"Received \"{trimmed_text}\" from {info_object.chat_id}", USER)
+                else:
+                    self.log(f"No Normal Message Handler defined", ERROR)
+        except Exception as e:
+            self.log(f"Error in Message_Handler: {e}", ERROR)
 
-    def loop(self, interval=0.3):
+    def loop(self, interval=0):
         while self._running:
             time.sleep(interval)
             updates = self.get_updates()
             if updates == None: continue
+            elif updates == requests.exceptions.ConnectionError:
+                print("[TELEASY] Connection abrupted. Trying again in 5 seconds")
+                time.sleep(10 - interval)
+                continue
             for update in updates:
                 if "message" in update:
                     self.handle_message_update(update)
@@ -407,13 +455,13 @@ class TelegramBot:
             self.log(str(args.exc_value), ERROR)
         threading.excepthook = thread_error_handler
 
-    def start(self, interval=0.3, console_logging=None):
+    def start(self, interval=0, console_logging=None):
         self._logging = False
         self.console_logging = console_logging if console_logging != None else self.console_logging
         self._config_error_handler()
         self.loop(interval=interval)
 
-    def start_with_interface(self, interval=0.3, end=True):
+    def start_with_interface(self, interval=0, end=True):
         self._config_error_handler()
         bot_thread = HandlerThread(target=lambda: self.loop(interval=interval))
         bot_thread.start()
@@ -502,4 +550,11 @@ class TelegramBot:
 
         Console.config(state=tkinter.DISABLED)
         window.mainloop()
-        self.active_interface = False
\ No newline at end of file
+        self.active_interface = False
+
+bot = TelegramBot("1826635110:AAFBZRZteyX0y5faukCkAeLVAsSW4EjFZ3E")
+bot.enable_console_logging()
+
+bot.set_command("hi", lambda info: "hallo!!!")
+
+bot.start()
\ No newline at end of file
